(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     60648,       1692]
NotebookOptionsPosition[     18451,        706]
NotebookOutlinePosition[     55879,       1525]
CellTagsIndexPosition[     55793,       1520]
WindowFrame->Normal
ContainsDynamic->False*)

(* Beginning of Notebook Content *)
Notebook[{
Cell["Evaluate Functions", "Title"],

Cell["\<\
Copyright \[Copyright] MathCore AB 1998-2003\
\>", "Copyright"],

Cell[CellGroupData[{

Cell["Initialization", "Section"],

Cell[CellGroupData[{

Cell["Load the MathCode package", "Subsection"],

Cell["\<\
First put a Needs statement, to make sure that the MathCode application is \
loaded:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<MathCode`\>\"", "]"}]], "Input"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"MathCode C++ 1.4.4\"\>", "\[InvisibleSpace]", "\<\" for \"\>", 
   "\[InvisibleSpace]", "\<\"mingw32\"\>", 
   "\[InvisibleSpace]", "\<\" loaded from \"\>", 
   "\[InvisibleSpace]", "\<\"C:\\\\b\\\\omath\\\\MathCode\"\>"}],
  SequenceForm[
  "MathCode C++ 1.4.4", " for ", "mingw32", " loaded from ", 
   "C:\\b\\omath\\MathCode"],
  Editable->False]], "Print",
 CellChangeTimes->{3.500374619524209*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Check current directory", "Subsection"],

Cell["\<\
Check the current directory, since a number of files will be placed there \
during the code generation process.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Directory", "[", "]"}]], "Input"],

Cell[BoxData["\<\"C:\\\\Documents and Settings\\\\vaden\\\\Mina \
dokument\"\>"], "Output",
 CellChangeTimes->{3.500374619602332*^9}]
}, Open  ]],

Cell["\<\
You might want to set the directory to some place where all generated files \
are put, e.g. the one below, or some other directory.\
\>", "Text"],

Cell["SetDirectory[$MCRoot<>\"/Demos/EvaluateFunctions\"];", "Input"],

Cell["Check whether this file is in the current directory.", "Text"],

Cell["ExistFileQ[\"EvaluateFunctions.nb\"]", "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Start of the EvaluateFunctions package", "Section"],

Cell["\<\
The EvaluateFunctions package starts in the usual way by a BeginPackage \
declaration which references other packages.The MathCodeContexts are needed \
in order to call the code generation related functions. \
\>", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"BeginPackage", "[", 
    RowBox[{"\"\<EvaluateFunctions`\>\"", ",", " ", 
     RowBox[{"{", "MathCodeContexts", "}"}]}], "]"}], ";"}], " "}], "\n", 
 RowBox[{
  RowBox[{"Off", "[", 
   RowBox[{"General", "::", "spell"}], "]"}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"Clear", "[", "\"\<EvaluateFunctions`*\>\"", "]"}], ";"}], 
  " "}]}], "Input",
 CellChangeTimes->{
  3.507524169651783*^9, {3.5075242916658463`*^9, 3.5075242923064632`*^9}, 
   3.507524364180543*^9}],

Cell[CellGroupData[{

Cell["Exported symbols", "Subsection"],

Cell["Define possibly exported symbols. ", "Text"],

Cell[CellGroupData[{

Cell["\<\
Begin[\"EvaluateFunctions`\"]
  var1; constvar1;
  vec1; constvec1;
  FunStandard; 
  FunEval; 
End[];\
\>", "Input"],

Cell[BoxData["\<\"EvaluateFunctions`\"\>"], "Output",
 CellChangeTimes->{3.500374619649206*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Setting compilation options", "Subsection"],

Cell["\<\
This defines how the functions and variables in the package should be \
compiled to C++. By default, all \
\>", "Text"],

Cell["\<\
SetCompilationOptions[
  EvaluateFunctions->{FunEval},
  MainFileAndFunction->\"int main(){return 0;}\"
];\
\>", "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["The body of the EvaluateFunctions package", "Subsection"],

Cell["\<\
Begin the implementation section of the EvaluateFunctions package, where \
functions are defined. This is usually private, to avoid accidental name \
shadowing due to identical local variables in several packages.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Begin", "[", "\"\<EvaluateFunctions`Private`\>\"", "]"}], 
  ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Global variables", "Subsection"],

Cell["Declare global variables.", "Text"],

Cell[TextData[{
 StyleBox["Var",
  FontColor->RGBColor[0, 0, 1]],
 "[\n  Constant Real[3]  constvec1 = 2;\n  Constant Real     constvar1 = 5;\n \
 ",
 StyleBox["Real[3]",
  FontColor->RGBColor[0, 0, 1]],
 "           vec1 = 3;\n  Real              var1 = 100;\n];"
}], "Input"],

Cell[CellGroupData[{

Cell[BoxData["constvec1"], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2.`", ",", "2.`", ",", "2.`"}], "}"}]], "Output",
 CellChangeTimes->{3.5003746196960807`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["vec1"], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3.`", ",", "3.`", ",", "3.`"}], "}"}]], "Output",
 CellChangeTimes->{3.500374619711705*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["var1"], "Input"],

Cell[BoxData["100"], "Output",
 CellChangeTimes->{3.50037461972733*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\n", 
  RowBox[{"constvec1", ".", "vec1"}]}]], "Input"],

Cell[BoxData["18.`"], "Output",
 CellChangeTimes->{3.5003746197585793`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["FunStandard", "Subsection"],

Cell["\<\
Compilation to a standard function, without symbolic expansion\
\>", "Text"],

Cell[TextData[{
 StyleBox["({Real,Real}->Real)",
  FontColor->RGBColor[0, 0, 1]],
 " ",
 StyleBox["FunStandard",
  FontColor->RGBColor[1, 0, 0]],
 "[x_, y_] :=  (\n    constvec1 . vec1 + 3 + constvar1 + var1 + x + y\n  );"
}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["FunEval", "Subsection"],

Cell["Function to be symbolically expanded. ", "Text"],

Cell[TextData[{
 StyleBox["({Real,Real}->Real)",
  FontColor->RGBColor[0, 0, 1]],
 " ",
 StyleBox["FunEval",
  FontColor->RGBColor[1, 0, 0]],
 "[x_, y_] :=  (\n    constvec1 . vec1 + 3 + constvar1 + var1 + x + y + \
If[x<y, 10, 20]\n  );"
}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["End of the EvaluateFunctions package", "Subsection"],

Cell["End[];", "Input"],

Cell["EndPackage[]; ", "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Execution", "Section"],

Cell[TextData[{
 "We first execute the application interpretively within ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", and then compile the key function and execute the application again. Then \
we compile the application to C++, build an executable, and call the same \
functions from ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " via ",
 StyleBox["MathLink",
  FontSlant->"Italic"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell["Fully Expanded Symbolic expression to be compiled to C++", "Subsection"],

Cell[TextData[{
 "This shows the size of ",
 StyleBox["FunEval",
  FontFamily->"Courier"],
 " fully expanded, with no evaluation protection of non-constant variables. "
}], "Text"],

Cell[CellGroupData[{

Cell["FunEval[aa, bb]", "Input"],

Cell[BoxData[
 RowBox[{"126.`", "\[VeryThinSpace]", "+", "aa", "+", "bb", "+", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"aa", "<", "bb"}], ",", "10", ",", "20"}], "]"}]}]], "Output",
 CellChangeTimes->{3.500374619789829*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DownValues", "[", "FunEval", "]"}], "[", 
  RowBox[{"[", "1", "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"HoldPattern", "[", 
   RowBox[{"FunEval", "[", 
    RowBox[{
    "EvaluateFunctions`Private`x_", ",", "EvaluateFunctions`Private`y_"}], 
    "]"}], "]"}], "\[RuleDelayed]", 
  RowBox[{
   RowBox[{"constvec1", ".", "vec1"}], "+", "3", "+", "constvar1", "+", 
   "var1", "+", "EvaluateFunctions`Private`x", "+", 
   "EvaluateFunctions`Private`y", "+", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{
     "EvaluateFunctions`Private`x", "<", "EvaluateFunctions`Private`y"}], ",",
      "10", ",", "20"}], "]"}]}]}]], "Output",
 CellChangeTimes->{3.5003746198054533`*^9}]
}, Open  ]],

Cell[TextData[{
 "This shows ",
 StyleBox["FunEval",
  FontFamily->"Courier"],
 " fully expanded, with evaluation protection of non-constant variables. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"funcSymbolicExpand", "[", 
  RowBox[{"Hold", "[", "FunEval", "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"funcSymbolicExpand", "[", 
  RowBox[{"Hold", "[", "FunEval", "]"}], "]"}]], "Output",
 CellChangeTimes->{3.5003746198210783`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["vec1"], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3.`", ",", "3.`", ",", "3.`"}], "}"}]], "Output",
 CellChangeTimes->{3.500374619836703*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Using the MathCode code generator", "Subsection"],

Cell["Compile the EvaluateFunctions package:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CompilePackage", "[", "\"\<EvaluateFunctions\>\"", "]"}]], "Input"],

Cell[BoxData["\<\"Successful compilation to C++: 2 function(s)\"\>"], "Print",
 CellChangeTimes->{3.500374622274156*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The generated C++ code", "Subsection"],

Cell[TextData[{
 "Below follows the generated C++ code from the EvaluateFunctions program. \
Notice that package name becomes a prefix of the name of the generated C++ \
function. Thus the Mathematica function ",
 StyleBox["EvaluateFunctions\[OpenCurlyQuote]sinFun2",
  FontFamily->"Courier"],
 " becomes ",
 StyleBox["EvaluateFunctions_TsinFun2",
  FontFamily->"Courier"],
 " in C++."
}], "Text"],

Cell["\<\
We give a command to type out the text of the generated C++ file:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FilePrint", "[", "\"\<EvaluateFunctions.cc\>\"", "]"}]], "Input"],

Cell["\<\
#include \"EvaluateFunctions.h\"

#include \"EvaluateFunctions.icc\"

#include <math.h>
void EvaluateFunctions_TEvaluateFunctionsInit ()
{
    EvaluateFunctions_Tvar1 = 100;
    EvaluateFunctions_Tvec1.Set(All,3.);; 
}

double EvaluateFunctions_TFunStandard ( const double &x, const double &y)
{
    return EvaluateFunctions_Tconstvec1*EvaluateFunctions_Tvec1+3+
        EvaluateFunctions_Tconstvar1+EvaluateFunctions_Tvar1+x+y;
}

double EvaluateFunctions_TFunEval ( const double &x, const double &y)
{
    double mc_T1;
    double mc_T2;
    double mc_T3;
    double mc_T4;
    double mc_T5;
    double mc_T6;
    int mc_T7;
    int mc_T8;
    double mc_T9;
    mc_T1 = EvaluateFunctions_Tvec1(3);
    mc_T2 = 2.*mc_T1;
    mc_T3 = EvaluateFunctions_Tvec1(2);
    mc_T4 = 2.*mc_T3;
    mc_T5 = EvaluateFunctions_Tvec1(1);
    mc_T6 = 2.*mc_T5;
    mc_T7 = x < y;
    if (mc_T7)
    {
        mc_T8 = 10;
    }
    else
    {
        mc_T8 = 20;
    }
    mc_T9 = 8+EvaluateFunctions_Tvar1+x+y+mc_T8+mc_T6+mc_T4+mc_T2;
    return mc_T9;
}

doubleN EvaluateFunctions_Tvec1(3);
double EvaluateFunctions_Tvar1;
doubleN EvaluateFunctions_Tconstvec1(3);
double EvaluateFunctions_Tconstvar1;\
\>", "Print",
 CellChangeTimes->{3.5003746223679047`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Compiling and linking the C++ code", "Subsection"],

Cell[TextData[{
 "The command ",
 StyleBox["MakeBinary",
  FontFamily->"Courier"],
 " compiles the generated C++ code using some C++ compiler (in this \
particular case the MicroSoft Visual C++, version 4.0, for Windows95). The \
object code is by default linked into two executables: ",
 StyleBox["EvaluateFunctionsml.exe",
  FontFamily->"Courier"],
 " for calling the compiled code via ",
 StyleBox["MathLink",
  FontSlant->"Italic"],
 ", and ",
 StyleBox["EvaluateFunctionsmain.exe",
  FontFamily->"Courier"],
 " as a stand-alone executable."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MakeBinary", "[", "]"}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Installing and connecting to Mathematica", "Subsection"],

Cell[TextData[{
 "The command ",
 StyleBox["InstallCode",
  FontFamily->"Courier"],
 " installs and connects the external process containing the compiled and \
linked EvaluateFunctions code."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"installedLink", "=", 
  RowBox[{"InstallCode", "[", "\"\<EvaluateFunctions\>\"", "]"}]}]], "Input"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"EvaluateFunctions\"\>", 
   "\[InvisibleSpace]", "\<\" is installed.\"\>"}],
  SequenceForm["EvaluateFunctions", " is installed."],
  Editable->False]], "Print",
 CellChangeTimes->{3.500374627492806*^9}],

Cell[BoxData[
 RowBox[{"LinkObject", "[", 
  RowBox[{"\<\"\\\"C:\\\\b\\\\omath\\\\MathCode\\\\Demos\\\\EvaluateFunctions\
\\\\EvaluateFunctionsml.exe\\\"\"\>", ",", "15", ",", "8"}], "]"}]], "Output",
 CellChangeTimes->{3.500374627492806*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Execution of generated C++ code", "Subsection"],

Cell["Perform the computation :", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FunStandard", "[", 
   RowBox[{"30.", ",", "40."}], "]"}], "  ", 
  RowBox[{"(*", " ", 
   RowBox[{"incorrect", ",", " ", 
    RowBox[{"since", " ", "global", " ", "vars", " ", "in", " ", 
     RowBox[{"c", "++"}], " ", "have", " ", "not", " ", "been", " ", 
     "initialized"}]}], " ", "*)"}]}]], "Input"],

Cell[BoxData["173.`"], "Output",
 CellChangeTimes->{3.5003746275240555`*^9}]
}, Open  ]],

Cell["Internal computation :", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunStandard", "[", 
  RowBox[{"30", ",", "40"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"FunStandard", "[", 
  RowBox[{"30", ",", "40"}], "]"}]], "Output",
 CellChangeTimes->{3.5003746275709295`*^9}]
}, Open  ]],

Cell["\<\
While the external version of the function is activated, the only valid \
values of the arguments are reals.\
\>", "Text"],

Cell["Perform the computation :", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FunEval", "[", 
   RowBox[{"30.", ",", "40."}], "]"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"correct", ",", " ", 
    RowBox[{"expanded", " ", "inline"}]}], " ", "*)"}]}]], "Input"],

Cell[BoxData["206.`"], "Output",
 CellChangeTimes->{3.500374627586554*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"??", "FunEval"}]], "Input"],

Cell["EvaluateFunctions`FunEval", "Print", "PrintUsage",
 CellChangeTimes->{3.5003746277271767`*^9},
 CellTags->"Info3500378227-4409646"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"ExternalDownValues", "[", "FunEval", "]"}], "^=", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"HoldPattern", "[", 
            RowBox[{"FunEval", "[", 
             RowBox[{
             "EvaluateFunctions`Private`x_Real", ",", 
              "EvaluateFunctions`Private`y_Real"}], "]"}], "]"}], 
           "\[RuleDelayed]", 
           RowBox[{"ExternalCall", "[", 
            RowBox[{
             RowBox[{"LinkObject", "[", 
              
              RowBox[{"\<\"\\\"C:\\\\b\\\\omath\\\\MathCode\\\\Demos\\\\\
EvaluateFunctions\\\\EvaluateFunctionsml.exe\\\"\"\>", ",", "15", ",", "8"}], 
              "]"}], ",", 
             RowBox[{"CallPacket", "[", 
              RowBox[{"2", ",", 
               RowBox[{"{", 
                RowBox[{
                "EvaluateFunctions`Private`x", ",", 
                 "EvaluateFunctions`Private`y"}], "}"}]}], "]"}]}], "]"}]}], 
          "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"SourceDownValues", "[", "FunEval", "]"}], "^=", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"HoldPattern", "[", 
            RowBox[{"FunEval", "[", 
             RowBox[{
             "EvaluateFunctions`Private`x_", ",", 
              "EvaluateFunctions`Private`y_"}], "]"}], "]"}], 
           "\[RuleDelayed]", 
           RowBox[{
            RowBox[{"constvec1", ".", "vec1"}], "+", "3", "+", "constvar1", 
            "+", "var1", "+", "EvaluateFunctions`Private`x", "+", 
            "EvaluateFunctions`Private`y", "+", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
              "EvaluateFunctions`Private`x", "<", 
               "EvaluateFunctions`Private`y"}], ",", "10", ",", "20"}], 
             "]"}]}]}], "}"}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]},
    {" "},
    {GridBox[{
       {
        RowBox[{
         RowBox[{"FunEval", "[", 
          RowBox[{
          "EvaluateFunctions`Private`x_Real", ",", 
           "EvaluateFunctions`Private`y_Real"}], "]"}], ":=", 
         RowBox[{"ExternalCall", "[", 
          RowBox[{
           RowBox[{"LinkObject", "[", 
            
            RowBox[{"\<\"\\\"C:\\\\b\\\\omath\\\\MathCode\\\\Demos\\\\\
EvaluateFunctions\\\\EvaluateFunctionsml.exe\\\"\"\>", ",", "15", ",", "8"}], 
            "]"}], ",", 
           RowBox[{"CallPacket", "[", 
            RowBox[{"2", ",", 
             RowBox[{"{", 
              RowBox[{
              "EvaluateFunctions`Private`x", ",", 
               "EvaluateFunctions`Private`y"}], "}"}]}], "]"}]}], "]"}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[EvaluateFunctions`FunEval],
  Editable->False]], "Print",
 CellChangeTimes->{3.500374627742801*^9},
 CellTags->"Info3500378227-4409646"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Clean Up", "Section"],

Cell["\<\
Uninstall[installedLink];
CleanMathCodeFiles[Confirm \[Rule] False,CleanAllBut \[Rule] {}]\
\>", "Input"]
}, Open  ]]
},
WindowToolbars->{},
WindowSize->{589, 896},
WindowMargins->{{45, Automatic}, {Automatic, 9}},
ShowSelection->True,
FrontEndVersion->"6.0 for Microsoft Windows (32-bit) (April 20, 2007)",
StyleDefinitions->Notebook[{
   Cell[
    CellGroupData[{
      Cell["Style Definitions", "Subtitle"], 
      Cell[
      "\nModify the definitions below to change the default appearance of all \
cells in \na given style.  Make modifications to any definition using \
commands in the \nFormat menu.\n", "Text"], 
      Cell[
       CellGroupData[{
         Cell["Style Environment Names", "Section"], 
         Cell[
          StyleData[All, "Working"], PageWidth -> WindowWidth, ScriptMinSize -> 
          9], 
         Cell[
          StyleData[All, "Presentation"], PageWidth -> WindowWidth, 
          ScriptMinSize -> 12, FontSize -> 16], 
         Cell[
          StyleData[All, "Condensed"], PageWidth -> WindowWidth, 
          CellBracketOptions -> {"Margins" -> {1, 1}, "Widths" -> {0, 5}}, 
          ScriptMinSize -> 8, FontSize -> 11], 
         Cell[
          StyleData[All, "Printout"], PageWidth -> PaperWidth, ScriptMinSize -> 
          7, FontSize -> 10, 
          PrivateFontOptions -> {"FontType" -> "Outline"}]}, Closed]], 
      Cell[
       CellGroupData[{
         Cell["Notebook Options", "Section"], 
         Cell[
         "\nThe options defined for the style below will be used at the \
Notebook level.\n", "Text"], 
         Cell[
          StyleData["Notebook"], PageHeaders -> {{
             Cell[
              TextData[{
                CounterBox["Page"]}], "PageNumber"], None, 
             Cell[
              TextData[{
                ValueBox["FileName"]}], "Header"]}, {
             Cell[
              TextData[{
                ValueBox["FileName"]}], "Header"], None, 
             Cell[
              TextData[{
                CounterBox["Page"]}], "PageNumber"]}}, 
          PageHeaderLines -> {True, True}, 
          PrintingOptions -> {
           "FacingPages" -> True, "FirstPageHeader" -> False}, 
          CellFrameLabelMargins -> 6, StyleMenuListing -> None]}, Closed]], 
      Cell[
       CellGroupData[{
         Cell["Styles for Headings", "Section"], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Title"], ShowCellBracket -> False, 
             CellMargins -> {{0, 0}, {0, 0}}, PageBreakBelow -> False, 
             LineSpacing -> {1, 0}, CounterIncrements -> "Title", 
             CounterAssignments -> {{"Section", 0}, {"Equation", 0}, {
               "Figure", 0}, {"Subtitle", 0}, {"Subsubtitle", 0}}, FontSize -> 
             34, FontColor -> GrayLevel[1], Background -> RGBColor[0, 0, 1]], 
            
            Cell[
             StyleData["Title", "Presentation"], 
             CellMargins -> {{0, 0}, {0, 0}}, LineSpacing -> {1, 0}, FontSize -> 
             44], 
            Cell[
             StyleData["Title", "Condensed"], CellMargins -> {{0, 0}, {0, 0}},
              FontSize -> 20], 
            Cell[
             StyleData["Title", "Printout"], CellMargins -> {{0, 0}, {0, 0}}, 
             FontSize -> 24, FontTracking -> "Plain", Background -> 
             GrayLevel[0]]}, Open]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Subtitle"], ShowCellBracket -> False, 
             CellMargins -> {{0, 0}, {0, 0}}, PageBreakBelow -> False, 
             LineSpacing -> {1, 3}, ParagraphIndent -> -96, CounterIncrements -> 
             "Subtitle", 
             CounterAssignments -> {{"Section", 0}, {"Equation", 0}, {
               "Figure", 0}, {"Subsubtitle", 0}}, FontFamily -> "Helvetica", 
             FontSize -> 18, FontColor -> GrayLevel[1], Background -> 
             RGBColor[0.2, 0.700008, 0.700008]], 
            Cell[
             StyleData["Subtitle", "Presentation"], 
             CellMargins -> {{0, 0}, {0, 0}}, LineSpacing -> {1, 6}, 
             ParagraphIndent -> -157, FontSize -> 30], 
            Cell[
             StyleData["Subtitle", "Condensed"], 
             CellMargins -> {{0, 0}, {0, 0}}, ParagraphIndent -> -78, 
             FontSize -> 14], 
            Cell[
             StyleData["Subtitle", "Printout"], 
             CellMargins -> {{0, 0}, {0, 0}}, ParagraphIndent -> -85, 
             FontSize -> 16, Background -> GrayLevel[0.6]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Subsubtitle"], ShowCellBracket -> False, 
             CellMargins -> {{10, 4}, {30, 10}}, PageBreakBelow -> False, 
             CounterIncrements -> "Subsubtitle", 
             CounterAssignments -> {{"Section", 0}, {"Equation", 0}, {
               "Figure", 0}}, FontFamily -> "Helvetica", FontSize -> 14, 
             FontSlant -> "Italic"], 
            Cell[
             StyleData["Subsubtitle", "Presentation"], 
             CellMargins -> {{8, 10}, {40, 20}}, LineSpacing -> {1, 0}, 
             FontSize -> 24], 
            Cell[
             StyleData["Subsubtitle", "Condensed"], 
             CellMargins -> {{8, 10}, {12, 8}}, FontSize -> 12], 
            Cell[
             StyleData["Subsubtitle", "Printout"], 
             CellMargins -> {{9, 10}, {50, 10}}, FontSize -> 14]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Section"], CellFrame -> {{6, 0}, {0, 1}}, CellDingbat -> 
             None, CellMargins -> {{12, Inherited}, {4, 24}}, 
             CellGroupingRules -> {"SectionGrouping", 30}, PageBreakBelow -> 
             False, CellFrameMargins -> 6, CounterIncrements -> "Section", 
             CounterAssignments -> {{"Subsection", 0}, {"Subsubsection", 0}}, 
             FontFamily -> "Helvetica", FontSize -> 16, FontWeight -> "Bold", 
             FontColor -> RGBColor[0.571389, 0.19675, 0.570504]], 
            Cell[
             StyleData["Section", "Presentation"], 
             CellMargins -> {{10, 10}, {8, 32}}, LineSpacing -> {1, 2}, 
             FontSize -> 24, FontTracking -> "Condensed"], 
            Cell[
             StyleData["Section", "Condensed"], 
             CellMargins -> {{8, Inherited}, {2, 12}}, FontSize -> 12], 
            Cell[
             StyleData["Section", "Printout"], 
             CellMargins -> {{9, 0}, {2, 50}}, FontSize -> 14, FontTracking -> 
             "Plain", FontColor -> GrayLevel[0]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Subsection"], 
             CellMargins -> {{12, Inherited}, {8, 20}}, 
             CellGroupingRules -> {"SectionGrouping", 40}, PageBreakBelow -> 
             False, CounterIncrements -> "Subsection", 
             CounterAssignments -> {{"Subsubsection", 0}}, FontFamily -> 
             "Helvetica", FontSize -> 13, FontWeight -> "Bold"], 
            Cell[
             StyleData["Subsection", "Presentation"], 
             CellMargins -> {{11, 10}, {8, 32}}, LineSpacing -> {1, 0}, 
             FontSize -> 22], 
            Cell[
             StyleData["Subsection", "Condensed"], 
             CellMargins -> {{8, Inherited}, {2, 12}}, FontSize -> 12], 
            Cell[
             StyleData["Subsection", "Printout"], 
             CellMargins -> {{9, 0}, {4, 40}}, FontSize -> 12]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Subsubsection"], CellDingbat -> "\[FilledSquare]", 
             CellMargins -> {{25, Inherited}, {8, 12}}, 
             CellGroupingRules -> {"SectionGrouping", 50}, PageBreakBelow -> 
             False, CounterIncrements -> "Subsubsection", FontFamily -> 
             "Times", FontSize -> 13, FontWeight -> "Bold"], 
            Cell[
             StyleData["Subsubsection", "Presentation"], 
             CellMargins -> {{29, 10}, {8, 26}}, LineSpacing -> {1, 0}, 
             FontSize -> 18], 
            Cell[
             StyleData["Subsubsection", "Condensed"], 
             CellMargins -> {{22, Inherited}, {2, 12}}, FontSize -> 10], 
            Cell[
             StyleData["Subsubsection", "Printout"], 
             CellMargins -> {{21, 0}, {4, 20}}, FontSize -> 11]}, Open]]}, 
        Closed]], 
      Cell[
       CellGroupData[{
         Cell["Styles for Body Text", "Section"], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Text"], CellMargins -> {{12, 10}, {5, 5}}, 
             LineSpacing -> {1, 3}, ParagraphSpacing -> {0, 12}, 
             CounterIncrements -> "Text", FontFamily -> "Times"], 
            Cell[
             StyleData["Text", "Presentation"], 
             CellMargins -> {{13, 10}, {8, 8}}, LineSpacing -> {1, 5}, 
             ParagraphSpacing -> {0, 12}], 
            Cell[
             StyleData["Text", "Condensed"], CellMargins -> {{8, 10}, {4, 4}},
              LineSpacing -> {1, 1}, ParagraphSpacing -> {0, 4}], 
            Cell[
             StyleData["Text", "Printout"], CellMargins -> {{9, 0}, {4, 4}}, 
             ParagraphSpacing -> {0, 6}]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["SmallText"], CellMargins -> {{12, 10}, {5, 5}}, 
             LineSpacing -> {1, 3}, ParagraphSpacing -> {0, 6}, 
             CounterIncrements -> "SmallText", FontFamily -> "Helvetica", 
             FontSize -> 9], 
            Cell[
             StyleData["SmallText", "Presentation"], 
             CellMargins -> {{13, 10}, {8, 8}}, LineSpacing -> {1, 5}, 
             FontSize -> 12], 
            Cell[
             StyleData["SmallText", "Condensed"], 
             CellMargins -> {{8, 10}, {2, 2}}, LineSpacing -> {1, 2}, 
             FontSize -> 9], 
            Cell[
             StyleData["SmallText", "Printout"], 
             CellMargins -> {{9, 0}, {4, 4}}, FontSize -> 7]}, Closed]]}, 
        Closed]], 
      Cell[
       CellGroupData[{
         Cell["Styles for Input/Output", "Section"], 
         Cell[
         "\nThe cells in this section define styles used for input and output \
to the \nkernel.  Be careful when modifying, renaming, or removing these \
styles, \nbecause the front end associates special meanings with these style \
names.\n", "Text"], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Input"], CellFrame -> {{3, 0}, {0, 0}}, 
             CellMargins -> {{52, 10}, {8, 8}}, Evaluatable -> True, 
             CellGroupingRules -> "InputGrouping", CellHorizontalScrolling -> 
             True, PageBreakWithin -> False, GroupPageBreakWithin -> False, 
             CellLabelMargins -> {{5, Inherited}, {Inherited, Inherited}}, 
             DefaultFormatType -> DefaultInputFormatType, FormatType -> 
             InputForm, ShowStringCharacters -> True, NumberMarks -> True, 
             CounterIncrements -> "Input", FontWeight -> "Bold", Background -> 
             RGBColor[1, 1, 0]], 
            Cell[
             StyleData["Input", "Presentation"], 
             CellMargins -> {{62, Inherited}, {10, 10}}, 
             LineSpacing -> {1, 0}], 
            Cell[
             StyleData["Input", "Condensed"], 
             CellMargins -> {{40, 10}, {4, 4}}], 
            Cell[
             StyleData["Input", "Printout"], CellMargins -> {{44, 0}, {6, 6}},
              Background -> GrayLevel[0.8]]}, Closed]], 
         Cell[
          StyleData["InlineInput"], Evaluatable -> True, CellGroupingRules -> 
          "InputGrouping", CellHorizontalScrolling -> True, PageBreakWithin -> 
          False, GroupPageBreakWithin -> False, DefaultFormatType -> 
          DefaultInputFormatType, AutoItalicWords -> {}, FormatType -> 
          InputForm, ShowStringCharacters -> True, NumberMarks -> True, 
          CounterIncrements -> "Input", FontWeight -> "Bold"], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Output"], CellFrame -> {{3, 0}, {0, 0}}, 
             CellMargins -> {{52, 10}, {8, 8}}, CellEditDuplicate -> True, 
             CellGroupingRules -> "OutputGrouping", CellHorizontalScrolling -> 
             True, PageBreakWithin -> False, GroupPageBreakWithin -> False, 
             GeneratedCell -> True, CellAutoOverwrite -> True, 
             CellLabelMargins -> {{3, Inherited}, {Inherited, Inherited}}, 
             DefaultFormatType -> DefaultOutputFormatType, FormatType -> 
             InputForm, CounterIncrements -> "Output", Background -> 
             RGBColor[1, 0.900008, 0.900008]], 
            Cell[
             StyleData["Output", "Presentation"], 
             CellMargins -> {{62, Inherited}, {12, 5}}, 
             LineSpacing -> {1, 0}], 
            Cell[
             StyleData["Output", "Condensed"], 
             CellMargins -> {{40, Inherited}, {4, 1}}], 
            Cell[
             StyleData["Output", "Printout"], 
             CellMargins -> {{44, 0}, {6, 2}}, Background -> 
             GrayLevel[0.900008]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Message"], 
             CellMargins -> {{62, Inherited}, {Inherited, Inherited}}, 
             CellGroupingRules -> "OutputGrouping", PageBreakWithin -> False, 
             GroupPageBreakWithin -> False, GeneratedCell -> True, 
             CellAutoOverwrite -> True, ShowCellLabel -> False, 
             DefaultFormatType -> DefaultOutputFormatType, FormatType -> 
             InputForm, CounterIncrements -> "Message", StyleMenuListing -> 
             None, FontColor -> RGBColor[1, 0, 0]], 
            Cell[
             StyleData["Message", "Presentation"], 
             CellMargins -> {{74, Inherited}, {Inherited, Inherited}}, 
             LineSpacing -> {1, 0}], 
            Cell[
             StyleData["Message", "Condensed"], 
             CellMargins -> {{50, Inherited}, {Inherited, Inherited}}], 
            Cell[
             StyleData["Message", "Printout"], 
             CellMargins -> {{54, Inherited}, {Inherited, Inherited}}, 
             FontColor -> GrayLevel[0]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Print"], 
             CellMargins -> {{62, Inherited}, {Inherited, Inherited}}, 
             CellGroupingRules -> "OutputGrouping", CellHorizontalScrolling -> 
             True, PageBreakWithin -> False, GroupPageBreakWithin -> False, 
             GeneratedCell -> True, CellAutoOverwrite -> True, ShowCellLabel -> 
             False, DefaultFormatType -> DefaultOutputFormatType, FormatType -> 
             InputForm, CounterIncrements -> "Print", StyleMenuListing -> 
             None], 
            Cell[
             StyleData["Print", "Presentation"], 
             CellMargins -> {{74, Inherited}, {Inherited, Inherited}}, 
             LineSpacing -> {1, 0}], 
            Cell[
             StyleData["Print", "Condensed"], 
             CellMargins -> {{50, Inherited}, {Inherited, Inherited}}], 
            Cell[
             StyleData["Print", "Printout"], 
             CellMargins -> {{54, Inherited}, {Inherited, Inherited}}]}, 
           Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Graphics"], 
             CellMargins -> {{62, Inherited}, {Inherited, Inherited}}, 
             CellGroupingRules -> "GraphicsGrouping", CellHorizontalScrolling -> 
             True, PageBreakWithin -> False, GeneratedCell -> True, 
             CellAutoOverwrite -> True, ShowCellLabel -> False, 
             DefaultFormatType -> DefaultOutputFormatType, FormatType -> 
             InputForm, CounterIncrements -> "Graphics", StyleMenuListing -> 
             None], 
            Cell[
             StyleData["Graphics", "Presentation"], 
             CellMargins -> {{74, Inherited}, {Inherited, Inherited}}], 
            Cell[
             StyleData["Graphics", "Condensed"], 
             CellMargins -> {{52, Inherited}, {Inherited, Inherited}}, 
             ImageSize -> {175, 175}], 
            Cell[
             StyleData["Graphics", "Printout"], 
             CellMargins -> {{54, Inherited}, {Inherited, Inherited}}, 
             ImageSize -> {250, 250}]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["CellLabel"], StyleMenuListing -> None, FontFamily -> 
             "Helvetica", FontSize -> 11, FontWeight -> "Bold", FontColor -> 
             RGBColor[0.571389, 0.19675, 0.570504]], 
            Cell[
             StyleData["CellLabel", "Presentation"], FontSize -> 12], 
            Cell[
             StyleData["CellLabel", "Condensed"], FontSize -> 8], 
            Cell[
             StyleData["CellLabel", "Printout"], FontSize -> 8, FontColor -> 
             GrayLevel[0]]}, Closed]]}, Open]], 
      Cell[
       CellGroupData[{
         Cell["Unique Styles", "Section"], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Author"], ShowCellBracket -> False, 
             CellMargins -> {{10, 4}, {2, 10}}, FontSize -> 16, FontSlant -> 
             "Italic"], 
            Cell[
             StyleData["Author", "Presentation"], 
             CellMargins -> {{12, 10}, {2, 12}}, LineSpacing -> {1, 5}, 
             ParagraphSpacing -> {0, 12}, FontSize -> 20], 
            Cell[
             StyleData["Author", "Condensed"], 
             CellMargins -> {{8, 10}, {1, 4}}, LineSpacing -> {1, 1}, 
             ParagraphSpacing -> {0, 4}, FontSize -> 12], 
            Cell[
             StyleData["Author", "Printout"], 
             CellMargins -> {{9, 0}, {4, 12}}, ParagraphSpacing -> {0, 6}, 
             FontSize -> 14]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Copyright"], ShowCellBracket -> False, 
             CellMargins -> {{10, 10}, {40, 2}}, FontFamily -> "Helvetica", 
             FontSize -> 9], 
            Cell[
             StyleData["Copyright", "Presentation"], 
             CellMargins -> {{12, 10}, {50, 2}}, LineSpacing -> {1, 5}, 
             FontSize -> 12], 
            Cell[
             StyleData["Copyright", "Condensed"], 
             CellMargins -> {{8, 10}, {12, 1}}, LineSpacing -> {1, 2}, 
             FontSize -> 9], 
            Cell[
             StyleData["Copyright", "Printout"], 
             CellMargins -> {{9, 0}, {72, 4}}, FontSize -> 7]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Example"], CellMargins -> {{12, 10}, {5, 12}}, 
             LineSpacing -> {1, 3}, ParagraphSpacing -> {0, 12}, 
             CounterIncrements -> "Example", FontFamily -> "Times", 
             FontWeight -> "Bold"], 
            Cell[
             StyleData["Example", "Presentation"], 
             CellMargins -> {{18, 10}, {8, 20}}, LineSpacing -> {1, 5}, 
             ParagraphSpacing -> {0, 12}], 
            Cell[
             StyleData["Example", "Condensed"], 
             CellMargins -> {{8, 10}, {4, 8}}, LineSpacing -> {1, 1}, 
             ParagraphSpacing -> {0, 4}], 
            Cell[
             StyleData["Example", "Printout"], 
             CellMargins -> {{9, 0}, {4, 10}}, ParagraphSpacing -> {0, 6}]}, 
           Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Definition"], CellFrame -> {{3, 0}, {0, 0}}, 
             CellMargins -> {{52, 10}, {8, 8}}, PageBreakWithin -> False, 
             GroupPageBreakWithin -> False, 
             CellLabelMargins -> {{23, Inherited}, {Inherited, Inherited}}, 
             ShowStringCharacters -> True, CounterIncrements -> "Definition", 
             FontFamily -> "Helvetica", FontWeight -> "Bold", FontColor -> 
             GrayLevel[1], Background -> RGBColor[0.2, 0.700008, 0.700008]], 
            Cell[
             StyleData["Definition", "Presentation"], 
             CellMargins -> {{62, Inherited}, {5, 12}}, 
             LineSpacing -> {1, 0}], 
            Cell[
             StyleData["Definition", "Condensed"], 
             CellMargins -> {{40, 10}, {1, 4}}], 
            Cell[
             StyleData["Definition", "Printout"], 
             CellMargins -> {{44, 0}, {2, 6}}, Background -> GrayLevel[0.6]]},
            Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Theorem"], CellFrame -> {{3, 0}, {0, 0}}, 
             CellMargins -> {{52, 10}, {8, 8}}, PageBreakWithin -> False, 
             GroupPageBreakWithin -> False, 
             CellLabelMargins -> {{23, Inherited}, {Inherited, Inherited}}, 
             ShowStringCharacters -> True, CounterIncrements -> "Theorem", 
             FontFamily -> "Helvetica", FontWeight -> "Bold", FontColor -> 
             GrayLevel[1], Background -> 
             RGBColor[0.571389, 0.19675, 0.570504]], 
            Cell[
             StyleData["Theorem", "Presentation"], 
             CellMargins -> {{62, Inherited}, {5, 12}}, 
             LineSpacing -> {1, 0}], 
            Cell[
             StyleData["Theorem", "Condensed"], 
             CellMargins -> {{40, 10}, {1, 4}}], 
            Cell[
             StyleData["Theorem", "Printout"], 
             CellMargins -> {{44, 0}, {2, 6}}, Background -> GrayLevel[0.4]]},
            Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["ExerciseMain"], CellFrame -> {{6, 0}, {0, 1}}, 
             CellMargins -> {{12, Inherited}, {4, 20}}, 
             CellGroupingRules -> {"SectionGrouping", 30}, PageBreakBelow -> 
             False, CellFrameColor -> RGBColor[0.571389, 0.19675, 0.570504], 
             CounterIncrements -> "Subsection", 
             CounterAssignments -> {{"Subsubsection", 0}}, FontFamily -> 
             "Helvetica", FontSize -> 16, FontWeight -> "Bold", FontColor -> 
             RGBColor[0.571389, 0.19675, 0.570504]], 
            Cell[
             StyleData["ExerciseMain", "Presentation"], 
             CellMargins -> {{18, 10}, {8, 32}}, LineSpacing -> {1, 2}, 
             FontSize -> 24, FontTracking -> "Condensed"], 
            Cell[
             StyleData["ExerciseMain", "Condensed"], 
             CellMargins -> {{8, Inherited}, {2, 12}}, FontSize -> 12], 
            Cell[
             StyleData["ExerciseMain", "Printout"], 
             CellMargins -> {{9, 0}, {2, 50}}, CellFrameColor -> 
             GrayLevel[0.500008], FontSize -> 14, FontTracking -> "Plain", 
             FontColor -> GrayLevel[0]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Exercise"], CellDingbat -> "\[FilledDownTriangle]", 
             CellMargins -> {{23, Inherited}, {4, 18}}, 
             CellGroupingRules -> {"SectionGrouping", 50}, PageBreakBelow -> 
             False, CounterIncrements -> "Subsubsection", FontFamily -> 
             "Times", FontSize -> 13, FontWeight -> "Bold", FontColor -> 
             RGBColor[0.571389, 0.19675, 0.570504]], 
            Cell[
             StyleData["Exercise", "Presentation"], 
             CellMargins -> {{33, 10}, {8, 26}}, LineSpacing -> {1, 0}, 
             FontSize -> 18], 
            Cell[
             StyleData["Exercise", "Condensed"], 
             CellMargins -> {{17, Inherited}, {2, 12}}, FontSize -> 10], 
            Cell[
             StyleData["Exercise", "Printout"], 
             CellFrame -> {{0, 0}, {0.5, 0}}, CellDingbat -> None, 
             CellMargins -> {{9, 0}, {6, 20}}, FontSize -> 11, FontColor -> 
             GrayLevel[0]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["ExerciseText"], CellMargins -> {{62, 10}, {5, 5}}, 
             LineSpacing -> {1, 3}, ParagraphSpacing -> {0, 8}, 
             CounterIncrements -> "ExcersiceText", FontFamily -> "Times"], 
            Cell[
             StyleData["ExerciseText", "Presentation"], 
             CellMargins -> {{74, 10}, {8, 8}}, LineSpacing -> {1, 5}, 
             ParagraphSpacing -> {0, 12}], 
            Cell[
             StyleData["ExerciseText", "Condensed"], 
             CellMargins -> {{52, 10}, {2, 2}}, LineSpacing -> {1, 1}, 
             ParagraphSpacing -> {0, 4}], 
            Cell[
             StyleData["ExerciseText", "Printout"], 
             CellMargins -> {{54, 0}, {4, 4}}, ParagraphSpacing -> {0, 6}]}, 
           Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Itemize"], CellDingbat -> "\[FilledSmallCircle]", 
             CellMargins -> {{65, Inherited}, {8, 8}}, FontFamily -> "Times", 
             FontSize -> 13, FontWeight -> "Bold"], 
            Cell[
             StyleData["Itemize", "Presentation"], 
             CellMargins -> {{Inherited, 10}, {8, 12}}, LineSpacing -> {1, 0},
              FontSize -> 18], 
            Cell[
             StyleData["Itemize", "Condensed"], 
             CellMargins -> {{Inherited, Inherited}, {2, 12}}, FontSize -> 
             10], 
            Cell[
             StyleData["Itemize", "Printout"], 
             CellMargins -> {{Inherited, 0}, {4, 20}}, FontSize -> 11]}, 
           Open]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["DisplayCode"], CellFrame -> 2, 
             CellMargins -> {{52, 10}, {8, 8}}, CellHorizontalScrolling -> 
             True, PageBreakWithin -> True, 
             CellLabelMargins -> {{5, Inherited}, {Inherited, Inherited}}, 
             DefaultFormatType -> DefaultInputFormatType, FormatType -> 
             InputForm, ShowStringCharacters -> True, NumberMarks -> True, 
             FontWeight -> "Bold", Background -> RGBColor[1, 1, 0]], 
            Cell[
             StyleData["DisplayCode", "Presentation"], 
             CellMargins -> {{62, Inherited}, {10, 10}}, 
             LineSpacing -> {1, 0}], 
            Cell[
             StyleData["DisplayCode", "Condensed"], 
             CellMargins -> {{40, 10}, {4, 4}}], 
            Cell[
             StyleData["DisplayCode", "Printout"], 
             CellMargins -> {{44, 0}, {6, 6}}, Background -> GrayLevel[0.8]]},
            Closed]]}, Open]], 
      Cell[
       CellGroupData[{
         Cell["Formulas and Programming", "Section"], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["DisplayFormula"], CellMargins -> {{62, 10}, {2, 10}}, 
             CellHorizontalScrolling -> True, ScriptLevel -> 0, 
             SingleLetterItalics -> True, 
             UnderoverscriptBoxOptions -> {LimitsPositioning -> True}], 
            Cell[
             StyleData["DisplayFormula", "Presentation"], 
             CellMargins -> {{74, 10}, {2, 10}}, FontSize -> 18], 
            Cell[
             StyleData["DisplayFormula", "Condensed"], 
             CellMargins -> {{52, 10}, {2, 10}}, FontSize -> 10], 
            Cell[
             StyleData["DisplayFormula", "Printout"], 
             CellMargins -> {{54, 10}, {2, 10}}, FontSize -> 10]}, Open]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["ChemicalFormula"], CellMargins -> {{62, 10}, {2, 10}},
              AutoSpacing -> False, ScriptBaselineShifts -> {0.6, Automatic}, 
             ScriptLevel -> 1, SingleLetterItalics -> False, ZeroWidthTimes -> 
             True], 
            Cell[
             StyleData["ChemicalFormula", "Presentation"], 
             CellMargins -> {{74, 10}, {2, 10}}, FontSize -> 10], 
            Cell[
             StyleData["ChemicalFormula", "Condensed"], 
             CellMargins -> {{52, 10}, {2, 10}}, FontSize -> 10], 
            Cell[
             StyleData["ChemicalFormula", "Printout"], 
             CellMargins -> {{54, 10}, {2, 10}}, FontSize -> 10]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Program"], CellMargins -> {{12, 10}, {Inherited, 6}}, 
             FontFamily -> "Courier"], 
            Cell[
             StyleData["Program", "Presentation"], 
             CellMargins -> {{13, 30}, {Inherited, 4}}, FontSize -> 9.5], 
            Cell[
             StyleData["Program", "Condensed"], 
             CellMargins -> {{8, 10}, {Inherited, 4}}, FontSize -> 9.5], 
            Cell[
             StyleData["Program", "Printout"], 
             CellMargins -> {{9, 0}, {Inherited, 4}}, FontSize -> 9.5]}, 
           Closed]]}, Closed]], 
      Cell[
       CellGroupData[{
         Cell["Styles for Automatic Numbering", "Section"], 
         Cell[
         "\nThe following styles are useful for numbered equations, figures, \
etc.  They \nautomatically give the cell a FrameLabel containing a reference \
to a \nparticular counter, and also increment that counter.\n", "Text"], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["NumberedEquation"], 
             CellMargins -> {{62, 10}, {Inherited, Inherited}}, 
             CellFrameLabels -> {{None, 
                Cell[
                 TextData[{"(", 
                   CounterBox["NumberedEquation"], ")"}]]}, {None, None}}, 
             DefaultFormatType -> DefaultInputFormatType, CounterIncrements -> 
             "NumberedEquation", FormatTypeAutoConvert -> False], 
            Cell[
             StyleData["NumberedEquation", "Presentation"], 
             CellMargins -> {{74, 10}, {Inherited, Inherited}}], 
            Cell[
             StyleData["NumberedEquation", "Condensed"], 
             CellMargins -> {{52, 10}, {Inherited, Inherited}}], 
            Cell[
             StyleData["NumberedEquation", "Printout"], 
             CellMargins -> {{54, 0}, {Inherited, Inherited}}]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["NumberedFigure"], 
             CellMargins -> {{62, 145}, {Inherited, Inherited}}, 
             CellFrameLabels -> {{None, None}, {
                Cell[
                 TextData[{"Figure ", 
                   CounterBox["NumberedFigure"]}]], None}}, CounterIncrements -> 
             "NumberedFigure", 
             ImageMargins -> {{43, Inherited}, {Inherited, 0}}, 
             FormatTypeAutoConvert -> False], 
            Cell[
             StyleData["NumberedFigure", "Presentation"]], 
            Cell[
             StyleData["NumberedFigure", "Condensed"]], 
            Cell[
             StyleData["NumberedFigure", "Printout"]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["NumberedTable"], 
             CellMargins -> {{62, 145}, {Inherited, Inherited}}, 
             CellFrameLabels -> {{None, None}, {
                Cell[
                 TextData[{"Table ", 
                   CounterBox["NumberedTable"]}]], None}}, TextAlignment -> 
             Center, CounterIncrements -> "NumberedTable", 
             FormatTypeAutoConvert -> False], 
            Cell[
             StyleData["NumberedTable", "Presentation"]], 
            Cell[
             StyleData["NumberedTable", "Condensed"]], 
            Cell[
             StyleData["NumberedTable", "Printout"]]}, Open]]}, Closed]], 
      Cell[
       CellGroupData[{
         Cell["Styles for Headers and Footers", "Section"], 
         Cell[
          StyleData["Header"], CellMargins -> {{0, 0}, {4, 1}}, 
          StyleMenuListing -> None, FontFamily -> "Helvetica", FontSize -> 9, 
          FontSlant -> "Italic"], 
         Cell[
          StyleData["Footer"], CellMargins -> {{0, 0}, {0, 4}}, 
          StyleMenuListing -> None, FontFamily -> "Helvetica", FontSize -> 6],
          
         Cell[
          StyleData["PageNumber"], CellMargins -> {{0, 0}, {4, 1}}, 
          StyleMenuListing -> None, FontFamily -> "Helvetica", FontSize -> 9, 
          FontWeight -> "Bold"]}, Closed]], 
      Cell[
       CellGroupData[{
         Cell["Hyperlink Styles", "Section"], 
         Cell[
         "\nThe cells below define styles useful for making hypertext \
ButtonBoxes.  The \n\"Hyperlink\" style is for links within the same \
Notebook, or between \nNotebooks.\n", "Text"], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Hyperlink"], StyleMenuListing -> None, 
             ButtonStyleMenuListing -> Automatic, FontColor -> GrayLevel[1], 
             Background -> RGBColor[1, 0.4, 0], 
             ButtonBoxOptions -> {
              Active -> True, Appearance -> {Automatic, None}, 
               ButtonFunction :> (FrontEndExecute[{
                  FrontEnd`NotebookLocate[#2]}]& ), ButtonNote -> 
               ButtonData}], 
            Cell[
             StyleData["Hyperlink", "Presentation"]], 
            Cell[
             StyleData["Hyperlink", "Condensed"]], 
            Cell[
             StyleData["Hyperlink", "Printout"], FontColor -> GrayLevel[0], 
             Background -> GrayLevel[1]]}, Closed]], 
         Cell[
         "\nThe following styles are for linking automatically to the on-line \
help \nsystem.\n", "Text"], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["MainBookLink"], StyleMenuListing -> None, 
             ButtonStyleMenuListing -> Automatic, FontColor -> GrayLevel[1], 
             Background -> RGBColor[1, 0.4, 0], 
             ButtonBoxOptions -> {
              Active -> True, Appearance -> {Automatic, None}, 
               ButtonFunction :> (FrontEndExecute[{
                  FrontEnd`HelpBrowserLookup["MainBook", #]}]& )}], 
            Cell[
             StyleData["MainBookLink", "Presentation"]], 
            Cell[
             StyleData["MainBookLink", "Condensed"]], 
            Cell[
             StyleData["MainBookLink", "Printout"], FontColor -> GrayLevel[0],
              Background -> GrayLevel[1]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["AddOnsLink"], StyleMenuListing -> None, 
             ButtonStyleMenuListing -> Automatic, FontFamily -> "Courier", 
             FontColor -> GrayLevel[1], Background -> RGBColor[1, 0.4, 0], 
             ButtonBoxOptions -> {
              Active -> True, Appearance -> {Automatic, None}, 
               ButtonFunction :> (FrontEndExecute[{
                  FrontEnd`HelpBrowserLookup["AddOns", #]}]& )}], 
            Cell[
             StyleData["AddOnsLink", "Presentation"]], 
            Cell[
             StyleData["AddOnsLink", "Condensed"]], 
            Cell[
             StyleData["AddOnLink", "Printout"], FontColor -> GrayLevel[0], 
             Background -> GrayLevel[1]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["RefGuideLink"], StyleMenuListing -> None, 
             ButtonStyleMenuListing -> Automatic, FontFamily -> "Courier", 
             FontColor -> GrayLevel[1], Background -> RGBColor[1, 0.4, 0], 
             ButtonBoxOptions -> {
              Active -> True, Appearance -> {Automatic, None}, 
               ButtonFunction :> (FrontEndExecute[{
                  FrontEnd`HelpBrowserLookup["RefGuideLink", #]}]& )}], 
            Cell[
             StyleData["RefGuideLink", "Presentation"]], 
            Cell[
             StyleData["RefGuideLink", "Condensed"]], 
            Cell[
             StyleData["RefGuideLink", "Printout"], FontColor -> GrayLevel[0],
              Background -> GrayLevel[1]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["GettingStartedLink"], StyleMenuListing -> None, 
             ButtonStyleMenuListing -> Automatic, FontColor -> GrayLevel[1], 
             Background -> RGBColor[1, 0.4, 0], 
             ButtonBoxOptions -> {
              Active -> True, Appearance -> {Automatic, None}, 
               ButtonFunction :> (FrontEndExecute[{
                  FrontEnd`HelpBrowserLookup["GettingStarted", #]}]& )}], 
            Cell[
             StyleData["GettingStartedLink", "Presentation"]], 
            Cell[
             StyleData["GettingStartedLink", "Condensed"]], 
            Cell[
             StyleData["GettingStartedLink", "Printout"], FontColor -> 
             GrayLevel[0], Background -> GrayLevel[1]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["OtherInformationLink"], StyleMenuListing -> None, 
             ButtonStyleMenuListing -> Automatic, FontColor -> GrayLevel[1], 
             Background -> RGBColor[1, 0.4, 0], 
             ButtonBoxOptions -> {
              Active -> True, Appearance -> {Automatic, None}, 
               ButtonFunction :> (FrontEndExecute[{
                  FrontEnd`HelpBrowserLookup["OtherInformation", #]}]& )}], 
            Cell[
             StyleData["OtherInformationLink", "Presentation"]], 
            Cell[
             StyleData["OtherInformationLink", "Condensed"]], 
            Cell[
             StyleData["OtherInformationLink", "Printout"], FontColor -> 
             GrayLevel[0], Background -> GrayLevel[1]]}, Closed]]}, Closed]]},
      Open]]}, Visible -> False, FrontEndVersion -> 
  "6.0 for Microsoft Windows (32-bit) (April 20, 2007)", StyleDefinitions -> 
  "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Info3500378227-4409646"->{
  Cell[14491, 592, 137, 2, 42, "Print",
   CellTags->"Info3500378227-4409646"],
  Cell[14631, 596, 3598, 95, 379, "Print",
   CellTags->"Info3500378227-4409646"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Info3500378227-4409646", 55606, 1512}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[580, 21, 35, 0, 55, "Title"],
Cell[618, 23, 73, 2, 54, "Copyright"],
Cell[CellGroupData[{
Cell[716, 29, 33, 0, 54, "Section"],
Cell[CellGroupData[{
Cell[774, 33, 47, 0, 44, "Subsection"],
Cell[824, 35, 108, 3, 25, "Text"],
Cell[CellGroupData[{
Cell[957, 42, 74, 1, 48, "Input"],
Cell[1034, 45, 452, 10, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1535, 61, 45, 0, 44, "Subsection"],
Cell[1583, 63, 135, 3, 25, "Text"],
Cell[CellGroupData[{
Cell[1743, 70, 57, 1, 48, "Input"],
Cell[1803, 73, 133, 2, 47, "Output"]
}, Open  ]],
Cell[1951, 78, 155, 3, 43, "Text"],
Cell[2109, 83, 69, 0, 48, "Input"],
Cell[2181, 85, 68, 0, 25, "Text"],
Cell[2252, 87, 53, 0, 48, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2354, 93, 57, 0, 54, "Section"],
Cell[2414, 95, 232, 4, 43, "Text"],
Cell[2649, 101, 522, 15, 89, "Input"],
Cell[CellGroupData[{
Cell[3196, 120, 38, 0, 44, "Subsection"],
Cell[3237, 122, 50, 0, 25, "Text"],
Cell[CellGroupData[{
Cell[3312, 126, 127, 7, 133, "Input"],
Cell[3442, 135, 95, 1, 47, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[3586, 142, 49, 0, 44, "Subsection"],
Cell[3638, 144, 129, 3, 25, "Text"],
Cell[3770, 149, 131, 5, 99, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[3938, 159, 63, 0, 44, "Subsection"],
Cell[4004, 161, 237, 4, 43, "Text"],
Cell[4244, 167, 112, 3, 48, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4393, 175, 38, 0, 44, "Subsection"],
Cell[4434, 177, 41, 0, 25, "Text"],
Cell[4478, 179, 277, 8, 133, "Input"],
Cell[CellGroupData[{
Cell[4780, 191, 35, 0, 48, "Input"],
Cell[4818, 193, 133, 3, 47, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4988, 201, 30, 0, 48, "Input"],
Cell[5021, 203, 131, 3, 47, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5189, 211, 30, 0, 48, "Input"],
Cell[5222, 213, 71, 1, 47, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5330, 219, 79, 2, 69, "Input"],
Cell[5412, 223, 75, 1, 47, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[5536, 230, 33, 0, 44, "Subsection"],
Cell[5572, 232, 86, 2, 25, "Text"],
Cell[5661, 236, 235, 7, 82, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5933, 248, 29, 0, 44, "Subsection"],
Cell[5965, 250, 54, 0, 25, "Text"],
Cell[6022, 252, 251, 8, 82, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6310, 265, 58, 0, 44, "Subsection"],
Cell[6371, 267, 23, 0, 48, "Input"],
Cell[6397, 269, 31, 0, 48, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[6477, 275, 28, 0, 54, "Section"],
Cell[6508, 277, 420, 13, 61, "Text"],
Cell[CellGroupData[{
Cell[6953, 294, 78, 0, 44, "Subsection"],
Cell[7034, 296, 180, 5, 25, "Text"],
Cell[CellGroupData[{
Cell[7239, 305, 32, 0, 48, "Input"],
Cell[7274, 307, 230, 5, 47, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[7541, 317, 120, 3, 48, "Input"],
Cell[7664, 322, 605, 16, 124, "Output"]
}, Open  ]],
Cell[8284, 341, 165, 5, 25, "Text"],
Cell[CellGroupData[{
Cell[8474, 350, 108, 2, 48, "Input"],
Cell[8585, 354, 153, 3, 47, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[8775, 362, 30, 0, 48, "Input"],
Cell[8808, 364, 131, 3, 47, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[8988, 373, 55, 0, 44, "Subsection"],
Cell[9046, 375, 54, 0, 25, "Text"],
Cell[CellGroupData[{
Cell[9125, 379, 91, 1, 48, "Input"],
Cell[9219, 382, 120, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[9388, 389, 44, 0, 44, "Subsection"],
Cell[9435, 391, 397, 10, 61, "Text"],
Cell[9835, 403, 89, 2, 25, "Text"],
Cell[CellGroupData[{
Cell[9949, 409, 89, 1, 48, "Input"],
Cell[10041, 412, 1255, 53, 823, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[11345, 471, 56, 0, 44, "Subsection"],
Cell[11404, 473, 557, 16, 79, "Text"],
Cell[11964, 491, 76, 2, 48, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[12077, 498, 62, 0, 44, "Subsection"],
Cell[12142, 500, 203, 6, 43, "Text"],
Cell[CellGroupData[{
Cell[12370, 510, 123, 2, 48, "Input"],
Cell[12496, 514, 252, 6, 23, "Print"],
Cell[12751, 522, 242, 4, 67, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[13042, 532, 53, 0, 44, "Subsection"],
Cell[13098, 534, 41, 0, 25, "Text"],
Cell[CellGroupData[{
Cell[13164, 538, 342, 8, 69, "Input"],
Cell[13509, 548, 76, 1, 47, "Output"]
}, Open  ]],
Cell[13600, 552, 38, 0, 25, "Text"],
Cell[CellGroupData[{
Cell[13663, 556, 89, 2, 48, "Input"],
Cell[13755, 560, 134, 3, 47, "Output"]
}, Open  ]],
Cell[13904, 566, 132, 3, 25, "Text"],
Cell[14039, 571, 41, 0, 25, "Text"],
Cell[CellGroupData[{
Cell[14105, 575, 218, 6, 48, "Input"],
Cell[14326, 583, 74, 1, 47, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[14437, 589, 51, 1, 48, "Input"],
Cell[14491, 592, 137, 2, 42, "Print",
 CellTags->"Info3500378227-4409646"],
Cell[14631, 596, 3598, 95, 379, "Print",
 CellTags->"Info3500378227-4409646"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[18290, 698, 27, 0, 54, "Section"],
Cell[18320, 700, 115, 3, 65, "Input"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
